#include "Riostream.h"

/* This program consists of two parts. Part one (treeFill) creates the histograms and part two (treeHist) draws them. 
   In part one the xlow, xhigh, bins, and nuevents parameters will need to be set. 
   In part two the fitmin and fitmax parameters will need to be set.   
   
   To run the program
   root [0] .x tree.C 

   To run part one by itself 
   root [0] .L tree.C 
   root [1] treeFill()

   To run part two by itself
   root [0] .L tree.C
   root [1] treeHist()

*/


void treeFill()
{
  gROOT->Reset();                       // Reset ROOT

  ifstream in;

  Float_t r1ch1,r1ch2,r1ch3,r1ch4,r1ch5,r1ch6,r1ch7,r1ch8,r1ch9,r1ch10,r2ch1,r2ch2,r3ch1,r3ch2,r3ch3,r3ch4,r3ch5,r3ch6,r3ch7,r3ch8,r3ch9,r3ch10,r4ch1,r4ch2,r5ch1,r5ch2,r5ch3,r5ch4,r6ch1,r6ch2,r6ch3,r6ch4;
  char line[256];

  // Open data file
  in.open("run363.dat"); 

  int numevents = 5000;                 // Number of events
  int xmean = 1763;
  int ymean = 1393;
  float slope = 0.01905;
  int yint = 1427;
  float corrected = 0;
  float dxeq;
  float xeq;

  int h1xlow = 1730;                      // Min histogram value to be graphed
  int h1xhigh = 1830;                     // Max histogram value to be graphed
  int h1xbins = (h1xhigh - h1xlow) / 1;        // Number of histogram bins 
 
  int h2xlow = 1350;                      // Min histogram value to be graphed
  int h2xhigh = 1450;                     // Max histogram value to be graphed
  int h2xbins = (h2xhigh - h2xlow) / 1;        // Number of histogram bins 

  int h3xlow = 0;                      // Min histogram value to be graphed
  int h3xhigh = numevents;                     // Max histogram value to be graphed
  int h3ylow = h1xlow;                      // Min histogram value to be graphed
  int h3yhigh = h1xhigh;                     // Max histogram value to be graphed
  int h3xbins = (h3xhigh - h3xlow) / 1;        // Number of histogram bins 
  int h3ybins = (h3yhigh - h3ylow) / 1;        // Number of histogram bins 

  int h4xlow = h1xlow;                      // Min histogram value to be graphed
  int h4xhigh = h1xhigh;                     // Max histogram value to be graphed
  int h4ylow = h2xlow;                      // Min histogram value to be graphed
  int h4yhigh = h2xhigh;                     // Max histogram value to be graphed
  int h4xbins = h1xbins;      		  // Number of histogram bins 
  int h4ybins = h2xbins;	        	// Number of histogram bins 

  int h5xlow = 1700;                      // Min histogram value to be graphed
  int h5xhigh = 1850;                     // Max histogram value to be graphed
  int h5xbins = (h5xhigh - h5xlow) / 1;        // Number of histogram bins 

  // Create a root file
  TFile *f = new TFile("test.root","RECREATE"); 

  // Create a 1D Histogram for the TAC data
  TH1F *h1 = new TH1F("h1","MCP Timing Resolution",h1xbins,h1xlow,h1xhigh);

  // Create a 1D Histogram for the TAC data
  TH1F *h2 = new TH1F("h2","MCP Pulse X Projection",h2xbins,h2xlow,h2xhigh);

  // Create a 2D Histogram for the Drift plot
  TH2F *h3 = new TH2F("h3","MCP Drift",h3xbins,h3xlow,h3xhigh,h3ybins,h3ylow,h3yhigh);

  // Create a 2D Histogram for the Correlation Plot
  TH2F *h4 = new TH2F("h4","MCP TAC vs Pulse X Projection",h4xbins,h4xlow,h4xhigh,h4ybins,h4ylow,h4yhigh);

  // Create a TProfile plot of ADC1 vs ADC2
  TProfile *hprof = new TProfile("hprof","Profile of TAC vs Pulse X Projection",h1xbins,h1xlow,h1xhigh);

  // Create a 1D Histogram for the TAC data
  TH1F *h5 = new TH1F("h5","MCP Corrected Timing Resolution",h5xbins,h5xlow,h5xhigh);

  // Create an Ntuple
  // r1ch1: row 1 channel 1 in your data file.
  TNtuple *ntuple = new TNtuple("ntuple","MCP Data","r1ch1:r1ch2:r1ch3:r1ch4:r1ch5:r1ch6:r1ch7:r1ch8:r1ch9:r1ch10:r2ch1:r2ch2:r3ch1:r3ch2:r3ch3:r3ch4:r3ch5:r3ch6:r3ch7:r3ch8:r3ch9:r3ch10:r4ch1:r4ch2:r5ch1:r5ch2:r5ch3:r5ch4:r6ch1:r6ch2:r6ch3:r6ch4");

  // Get the first line w/ run information
  in.getline (line,256);

  // Loop over events
  int i = 1;   
  while (1)
    {
      // Row 1
      in >> r1ch1 >> r1ch2 >> r1ch3 >> r1ch4 >> r1ch5 >> r1ch6 >> r1ch7 >> r1ch8 >> r1ch9 >> r1ch10;
      if (!in.good()) break;
      h1->Fill(r1ch1);
      h3->Fill(i,r1ch1);
	h2->Fill(r1ch2);
	h4->Fill(r1ch1,r1ch2);
	hprof->Fill(r1ch1,r1ch2);

	xeq = (r1ch2 - yint) / slope;
	dxeq = xmean - xeq;
      corrected = r1ch1 + dxeq;
	h5->Fill(corrected);

      ntuple->Fill(r1ch1,r1ch2,r1ch3,r1ch4,r1ch5,r1ch6,r1ch7,r1ch8,r1ch9,r1ch10);

      // Row 2
      in >> r2ch1 >> r2ch2;
      if (!in.good()) break;
      //ntuple->Fill(r2ch1,r2ch2); If there is useful info in this row remove comment

      //Row 3
      in >> r3ch1 >> r3ch2 >> r3ch3 >> r3ch4 >> r3ch5 >> r3ch6 >> r3ch7 >> r3ch8 >> r3ch9 >> r3ch10;
      if (!in.good()) break;
      //ntuple->Fill(r3ch1,r3ch2,r3ch3,r3ch4,r3ch5,r3ch6,r3ch7,r3ch8,r3ch9,r3ch10); If there is useful info in this row remove comment

      // Row 4
      in >> r4ch1 >> r4ch2;
      if (!in.good()) break;
      //ntuple->Fill(r4ch1,r4ch2); If there is useful info in this row remove comment
 
      // Row 5
      in >> r5ch1 >> r5ch2 >> r5ch3 >> r5ch4;
      if (!in.good()) break;
      //ntuple->Fill(r5ch1,r5ch2,r5ch3,r5ch4); If there is useful info in this row remove comment
 
      // Row 6
      in >> r6ch1 >> r6ch2 >> r6ch3 >> r6ch4;
      if (!in.good()) break;
      //ntuple->Fill(r6ch1,r6ch2,r6ch3,r6ch4); If there is useful info in this row remove comment

      i++;
    }

  printf( "Found %i points\n",i);

  in.close(); // Close the data file

  f->Write(); // Write the root file
  //f->Close(); // Close the root file

}

void treeCorrected()
{
  extern TStyle* gStyle;

  int fitmin1 = 1752;
  int fitmax1 = 1774;
  int fitmin2 = 1388;
  int fitmax2 = 1398;
  int fitmin3 = 1740;
  int fitmax3 = 1762;
  
  TFile *f = new TFile("test.root");                    // Open the root file
  TAC = (TH1F*)f->Get("h1;1");                          // Get the 1D histogram
  COR_TAC = (TH1F*)f->Get("h5;1");                          // Get the 1D histogram
  SLOPE = (TH1F*)f->Get("h2;1");                          // Get the 1D histogram
  TACvsSLOPE = (TH2F*)f->Get("h4;1");                       // Get the 2D histogram
  PROFILE = (TProfile*)f->Get("hprof;1");

  //**************************
  //  ROOT Plotting Options
  //**************************
  // Set histogram plotting options
  gStyle->SetOptStat(2211);
  gStyle->SetOptFit(11);
  gStyle->SetFuncColor( 1 );
  gStyle->SetFrameFillColor( 10 );
  //gStyle->SetHistFillColor( 3 );
  gStyle->SetLabelSize( 0.03, "XYZ" );
  gStyle->SetStatColor( 0 );
  gStyle->SetStatFontSize( 0.03 );
  gStyle->SetTitleColor( 2 );
  gStyle->SetTitleW(0.5);
  //gStyle->SetMarkerColor( 6 );
  //gStyle->SetMarkerSize( 0.25 );
  //gStyle->SetMarkerStyle( 8 );
  gStyle->SetPaperSize(20,20);
  

  TCanvas *MyC1 = new TCanvas("MyC1","Canvas",1200,800);   // Create a canvas for our graphs
  MyC1->Divide(2,2);                                     // Divide the canvas into two pads

  MyC1->cd(1);               // pad 1
  TAC->SetXTitle( "ADC Counts");
  TAC->GetXaxis()->SetTitleColor( 2 );
  TAC->SetYTitle( "Events");
  TAC->GetYaxis()->SetTitleColor( 2 );
  TAC->GetYaxis()->SetTitleOffset( 1.2 );
  TF1* myfit = new TF1("myfit","gaus",fitmin1,fitmax1);
  myfit->SetParName(0,"Norm");
  myfit->SetParName(1,"Mean");
  myfit->SetParName(2,"Sigma");
  TAC->Draw();              // Draw the 1D histogram
  TAC->Fit("myfit","R");  // use this to fit over a range and comment out previous

  MyC1->cd(4);               // pad 2
  COR_TAC->SetXTitle( "ADC Counts");
  COR_TAC->GetXaxis()->SetTitleColor( 2 );
  COR_TAC->SetYTitle( "Events");
  COR_TAC->GetYaxis()->SetTitleColor( 2 );
  COR_TAC->GetYaxis()->SetTitleOffset( 1.2 );
  TF1* myfit = new TF1("myfit","gaus",fitmin3,fitmax3);
  myfit->SetParName(0,"Norm");
  myfit->SetParName(1,"Mean");
  myfit->SetParName(2,"Sigma");
  COR_TAC->Draw();              // Draw the 1D histogram
  COR_TAC->Fit("myfit","R");  // use this to fit over a range and comment out previous

  MyC1->cd(2);               // pad 1
  SLOPE->SetXTitle( "ADC Counts");
  SLOPE->GetXaxis()->SetTitleColor( 2 );
  SLOPE->SetYTitle( "Events");
  SLOPE->GetYaxis()->SetTitleColor( 2 );
  SLOPE->GetYaxis()->SetTitleOffset( 1.2 );
  TF1* myfit = new TF1("myfit","gaus",fitmin2,fitmax2);
  myfit->SetParName(0,"Norm");
  myfit->SetParName(1,"Mean");
  myfit->SetParName(2,"Sigma");
  SLOPE->Draw();              // Draw the 1D histogram
  SLOPE->Fit("myfit","R");  // use this to fit over a range and comment out previous

  MyC1->cd(3);               // pad 2
  TACvsSLOPE->SetStats(kFALSE);
  TACvsSLOPE->SetXTitle( "TAC");
  TACvsSLOPE->GetXaxis()->SetTitleColor( 2 );
  TACvsSLOPE->SetYTitle( "X Projection");
  TACvsSLOPE->GetYaxis()->SetTitleColor( 2 );
  TACvsSLOPE->GetYaxis()->SetTitleOffset( 1.2 );
  TACvsSLOPE->Draw("SCAT");     // Draw the 2D histogram as a scatter plot

  TCanvas *MyC2 = new TCanvas("MyC2","Canvas",300,200,600,400);   // Create a canvas for our graphs
  PROFILE->SetXTitle( "TAC");
  PROFILE->GetXaxis()->SetTitleColor( 2 );
  PROFILE->SetYTitle( "SLOPE");
  PROFILE->GetYaxis()->SetTitleColor( 2 );
  PROFILE->GetYaxis()->SetTitleOffset( 1.2 );
  PROFILE->Draw();


}


void treeHist()
{
  extern TStyle* gStyle;

  int fitmin1 = 1760;
  int fitmax1 = 1778;
  int fitmin2 = 1454;
  int fitmax2 = 1490;

  TFile *f = new TFile("test.root");                    // Open the root file
  TAC = (TH1F*)f->Get("h1;1");                          // Get the 1D histogram
  EVENTS = (TH2F*)f->Get("h3;1");                       // Get the 2D histogram
  SLOPE = (TH1F*)f->Get("h2;1");                          // Get the 1D histogram
  TACvsSLOPE = (TH2F*)f->Get("h4;1");                       // Get the 2D histogram
  PROFILE = (TProfile*)f->Get("hprof;1");

  //**************************
  //  ROOT Plotting Options
  //**************************
  // Set histogram plotting options
  gStyle->SetOptStat(2211);
  gStyle->SetOptFit(11);
  gStyle->SetFuncColor( 1 );
  gStyle->SetFrameFillColor( 10 );
  //gStyle->SetHistFillColor( 3 );
  gStyle->SetLabelSize( 0.03, "XYZ" );
  gStyle->SetStatColor( 0 );
  gStyle->SetStatFontSize( 0.03 );
  gStyle->SetTitleColor( 2 );
  gStyle->SetTitleW(0.5);
  //gStyle->SetMarkerColor( 6 );
  //gStyle->SetMarkerSize( 0.25 );
  //gStyle->SetMarkerStyle( 8 );
  gStyle->SetPaperSize(20,20);
  

  TCanvas *MyC1 = new TCanvas("MyC1","Canvas",600,800);   // Create a canvas for our graphs
  MyC1->Divide(1,2);                                     // Divide the canvas into two pads

  MyC1->cd(1);               // pad 1
  TAC->SetXTitle( "MCP Timing Resolution            ADC Counts");
  TAC->GetXaxis()->SetTitleColor( 2 );
  TAC->SetYTitle( "Events");
  TAC->GetYaxis()->SetTitleColor( 2 );
  TAC->GetYaxis()->SetTitleOffset( 1.2 );
  TF1* myfit = new TF1("myfit","gaus",fitmin1,fitmax1);
  myfit->SetParName(0,"Norm");
  myfit->SetParName(1,"Mean");
  myfit->SetParName(2,"Sigma");
  // myfit->SetParameter(1, mean);
  // myfit->SetParameter(2, rms);
  TAC->Draw();              // Draw the 1D histogram
  //  TAC->Fit("gaus");
  TAC->Fit("myfit","R");  // use this to fit over a range and comment out previous

  MyC1->cd(2);               // pad 2
  EVENTS->SetStats(kFALSE);
  EVENTS->SetXTitle( "MCP Drift            Event Number");
  EVENTS->GetXaxis()->SetTitleColor( 2 );
  EVENTS->SetYTitle( "ADC Counts");
  EVENTS->GetYaxis()->SetTitleColor( 2 );
  EVENTS->GetYaxis()->SetTitleOffset( 1.2 );
  EVENTS->Draw("SCAT");     // Draw the 2D histogram as a scatter plot

  TCanvas *MyC2 = new TCanvas("MyC2","Canvas",610,0,600,800);   // Create a canvas for our graphs
  MyC2->Divide(1,2);                                     // Divide the canvas into two pads

  MyC2->cd(1);               // pad 1
  SLOPE->SetXTitle( "MCP Slope            ADC Counts");
  SLOPE->GetXaxis()->SetTitleColor( 2 );
  SLOPE->SetYTitle( "Events");
  SLOPE->GetYaxis()->SetTitleColor( 2 );
  SLOPE->GetYaxis()->SetTitleOffset( 1.2 );
  TF1* myfit = new TF1("myfit","gaus",fitmin2,fitmax2);
  myfit->SetParName(0,"Norm");
  myfit->SetParName(1,"Mean");
  myfit->SetParName(2,"Sigma");
  // myfit->SetParameter(1, mean);
  // myfit->SetParameter(2, rms);
  SLOPE->Draw();              // Draw the 1D histogram
  //  SLOPE->Fit("gaus");
  SLOPE->Fit("myfit","R");  // use this to fit over a range and comment out previous

  MyC2->cd(2);               // pad 2
  TACvsSLOPE->SetStats(kFALSE);
  TACvsSLOPE->SetXTitle( "MCP TAC vs SLOPE            TAC");
  TACvsSLOPE->GetXaxis()->SetTitleColor( 2 );
  TACvsSLOPE->SetYTitle( "SLOPE");
  TACvsSLOPE->GetYaxis()->SetTitleColor( 2 );
  TACvsSLOPE->GetYaxis()->SetTitleOffset( 1.2 );
  TACvsSLOPE->Draw("SCAT");     // Draw the 2D histogram as a scatter plot

  TCanvas *MyC3 = new TCanvas("MyC3","Canvas",300,200,600,400);   // Create a canvas for our graphs

  PROFILE->SetXTitle( "MCP TAC vs SLOPE Profile            TAC");
  PROFILE->GetXaxis()->SetTitleColor( 2 );
  PROFILE->SetYTitle( "SLOPE");
  PROFILE->GetYaxis()->SetTitleColor( 2 );
  PROFILE->GetYaxis()->SetTitleOffset( 1.2 );
  PROFILE->Draw();

}


void tree()
{
  treeFill();
  treeCorrected();
}
